package de.hs_mannheim.informatik.bank.facade;

import java.util.Collection;

import de.hs_mannheim.informatik.bank.domain.Bank;
import de.hs_mannheim.informatik.bank.domain.Depot;
import de.hs_mannheim.informatik.bank.domain.Girokonto;
import de.hs_mannheim.informatik.bank.domain.Konto;
import de.hs_mannheim.informatik.bank.domain.Kunde;
import de.hs_mannheim.informatik.bank.infrastructure.Persistenz;

public class Banksystem {
	private Bank bank;

	public Banksystem(String bankname) throws Exception {
		if (Persistenz.sindDatenGespeichert(bankname))
			this.bank = (Bank) Persistenz.ladeBankDaten(bankname);
		else
			this.bank = new Bank(bankname);
	}

	public int kontoAnlegen(Kunde kunde, int auswahl) throws Exception {
		int kontonummer = bank.addKonto(kunde, auswahl);
		//if (kontonummer == -1) {
		//	throw new Exception("Ungültige Auswahl");

		//}

		Persistenz.speichereBankDaten(this.bank, bank.getName());

		return kontonummer;
	}

	public String[] getKontenliste() {
		Collection<Konto> konten = bank.getKontenliste();
		String[] liste = new String[konten.size()];

		int i = 0;
		for (Konto k : konten) {
			liste[i++] = k.toString();
		}

		return liste;
	}

	public String[] getKundenliste() {
		Collection<Kunde> kunden = bank.getKundenliste();
		String[] liste = new String[kunden.size()];

		int i = 0;
		for (Kunde k : kunden) {
			liste[i++] = k.toString();
		}

		return liste;
	}
	public String[] getDepotliste() {
		Collection<Depot> depots = bank.getDepotliste();
		String[] liste = new String[depots.size()];

		int i = 0;
		for (Depot d : depots) {
			liste[i++] = d.toString();
		}

		return liste;
	}

	public String getBankname() {
		return bank.getName();
	}

	public long geldEinzahlen(int kontonummer, long betrag) throws Exception {
		Konto konto = bank.findeKonto(kontonummer);
		konto.einzahlen(betrag, "Einzahlung am Schalter", "Einzahlung", konto.getInhaber());

		Persistenz.speichereBankDaten(this.bank, bank.getName());

		return konto.getKontostand();
	}

	public boolean geldAuszahlen(int kontonummer, long betrag) throws Exception {
		Konto konto = bank.findeKonto(kontonummer);

		boolean erg = konto.auszahlen(betrag, "Auszahlung am Schalter", "Auszahlung", konto.getInhaber());

		Persistenz.speichereBankDaten(this.bank, bank.getName());

		return erg;
	}

	public String[] erstelleKontoauszug(int kontonummer) {
		Konto konto = bank.findeKonto(kontonummer);

		return konto.getKontobewegungen();
	}

	public boolean überweisungBeauftragen(int startkonto, int zielkonto, long betrag, String verwendungszweck)
			throws Exception {
		Konto start = bank.findeKonto(startkonto);
		Konto ziel = bank.findeKonto(zielkonto);

		if (start instanceof Girokonto && ziel instanceof Girokonto) {
			boolean erfolg = ((Girokonto) start).überweise((Girokonto) ziel, betrag, verwendungszweck);
			Persistenz.speichereBankDaten(this.bank, bank.getName());

			return erfolg;
		}

		return false;
	}

	public long getKontostand(int kontonummer) {
		Konto konto = bank.findeKonto(kontonummer);

		return konto.getKontostand();
	}

	public long saldoBestimmen(int kontonummer, int anzahl) {
		Konto konto = bank.findeKonto(kontonummer);

		return konto.berechneSaldo(anzahl);
	}

	public int kundeAnlegen(String vorname, String nachname, String wohnort, int alter) {
		int kundennummer = bank.addKunde(vorname, nachname, wohnort, alter);
		return kundennummer;

	}

	public Kunde findeKunde(int kundennummer) {
		Kunde k = bank.findeKunde(kundennummer);
		return k;

	}

	public boolean aktienKaufen(int kontonummer, int auswahl, int anzahl) {
		Depot depot = bank.findeDepot(kontonummer);
		boolean erfolgreich = depot.aktienKaufen(auswahl,anzahl);
		
		return erfolgreich;
		
	}

	public long geldEinzahlenDepot(int kontonummer, long betrag) {
		Depot depot = bank.findeDepot(kontonummer);
		depot.geldEinzahlen(betrag);
		return depot.getKontostand();
	}

	public boolean geldAuszahlenDepot(int kontonummer, long betrag) {
		Depot depot = bank.findeDepot(kontonummer);
		boolean erfolgreich = depot.geldAuszahlen(betrag);
		
		return erfolgreich;
	}

	public String[] aktienAnzeigen(int kontonummer) {
		Depot depot = bank.findeDepot(kontonummer);
		String [] aktien = depot.getAktien();
		
		return aktien;
		
	}

	public boolean aktienVerkaufen(int kontonummer, int auswahl, int anzahl) {
		Depot depot = bank.findeDepot(kontonummer);
		return depot.aktienVerkaufen(auswahl, anzahl);
	}

}
