package de.hs_mannheim.informatik.bank.ui;

import java.util.Scanner;

import de.hs_mannheim.informatik.bank.domain.Kunde;
import de.hs_mannheim.informatik.bank.facade.Banksystem;

public class UI {
	private Banksystem bs;
	Scanner sc = new Scanner(System.in);

	public UI(Banksystem bs) {
		this.bs = bs;
		hauptmenü();
	}

	private void hauptmenü() {
		System.out.println("Willkommen bei der " + bs.getBankname() + "!");

		mainloop: while (true) {
			System.out.println();
			System.out.println("--------");
			System.out.println("Hauptmenü");
			System.out.println("1 -> Konten anzeigen");
			System.out.println("2 -> Konto oder Depot anlegen");
			System.out.println("3 -> Geld einzahlen (Giro- oder Sparkonto)");
			System.out.println("4 -> Geld auszahlen (Giro- oder Sparkonto)");
			System.out.println("5 -> Geld auf ein Depot einzahlen ");
			System.out.println("6 -> Geld von einem Depot auszahlen");
			System.out.println("7 -> Kontoauszug drucken");
			System.out.println("8 -> Überweisung beauftragen");
			System.out.println("9 -> Saldo abfragen");
			System.out.println("10 -> Aktien kaufen");
			System.out.println("11 -> Aktien verkaufen");
			System.out.println("12 -> Aktien von einem Depot anzeigen");
			System.out.println("13 -> Kunde anlegen");
			System.out.println("14 -> Beenden");
			System.out.println();

			System.out.print("> ");
			int input = Integer.parseInt(sc.nextLine());
			System.out.println();

			try {
				switch (input) {
				case 1:
					kontenAnzeigen();
					break;
				case 2:
					kontoAnlegen();
					break;
				case 3:
					geldEinzahlen();
					break;
				case 4:
					geldAuszahlen();
					break;
				case 5:
					geldEinzahlennDepot();
					break;
				case 6:
					geldAuszahlenDepot();
					break;
				case 7:
					kontoauszugDrucken();
					break;
				case 8:
					überweisungBeauftragen();
					break;
				case 9:
					saldoAbfragen();
					break;
				case 10:
					aktienKaufen();
					break;
				case 11:
					aktienVerkaufen();
					break;
				case 12:
					aktienAnzeigen();
					break;
				case 13:
					kundeAnlegen();
					break;
				case 14:
					break mainloop;
				}

			} catch (Exception e) {
				System.err.println(e.getLocalizedMessage());
			}
			System.out.println();
		}

		System.out.println("Auf Wiedersehen!");

	} // hauptmenü

	private void kontenAnzeigen() {
		String[] konten = bs.getKontenliste();
		if (konten.length > 0) {
			System.out.println("Folgende Konten sind aktuell verfügbar:");
			for (String s : konten) {
				System.out.println(s);
			}
		} else {
			System.out.println("Bisher keine Konten angelegt.");
		}
		String[] depots = bs.getDepotliste();
		if (depots.length > 0) {
			System.out.println("Folgende Depots sind aktuell verfügbar:");
			for (String s : depots) {
				System.out.println(s);
			}
		} else {
			System.out.println("Bisher keine Depots angelegt.");
		}
	}

	private void kontoAnlegen() throws Exception {
		String[] kunden = bs.getKundenliste();
		if (kunden.length == 0) {
			System.out.println("Es gibt noch keine Kunden, bitte Kunde anlegen.");
			kundeAnlegen();
		}

		// System.out.println("Bitte den Namen des Kontoinhabers angeben: ");
		// String name = sc.nextLine();

		System.out.println("Möchten Sie ein Sparkonto (1), ein Girokonto (2) oder ein Wertpapier-Depot (3) anlegen?");
		int auswahl = Integer.parseInt(sc.nextLine());
		System.out.println("Bitte geben Sie eine Kundennummer ein: ");
		int kundennummer = Integer.parseInt(sc.nextLine());
		Kunde k = bs.findeKunde(kundennummer);
		if (k == null) {
			System.out.println("Kunde existiert nicht, bitte neu versuchen.");
			kontoAnlegen();
		} else {
			int kontonummer = bs.kontoAnlegen(k, auswahl);
			System.out.println("Konto mit der Nummer " + kontonummer + " neu angelegt.");
		}
	}

	private void geldEinzahlen() throws Exception {
		System.out.println("Geld einzahlen");
		System.out.print("Bitte die gewünschte Kontonummer eingeben: ");
		int kontonummer = Integer.parseInt(sc.nextLine());

		// optional prüfen, ob Konto existiert

		System.out.print("Bitte den gewünschten Betrag eingeben: ");
		double betrag = Double.parseDouble(sc.nextLine());

		long neuerKontostand = bs.geldEinzahlen(kontonummer, (long) betrag * 100);

		System.out.printf("Einzahlung erfolgreich, neuer Kontostand = %.2f Euro", (neuerKontostand / 100.0));
	}

	private void geldAuszahlen() throws Exception {
		System.out.println("Geld auszahlen");
		System.out.print("Bitte die gewünschte Kontonummer eingeben: ");
		int kontonummer = Integer.parseInt(sc.nextLine());

		System.out.print("Bitte den gewünschten Betrag eingeben: ");
		double betrag = Double.parseDouble(sc.nextLine());

		boolean erfolgreich = bs.geldAuszahlen(kontonummer, (long) betrag * 100);

		System.out.printf("Auszahlung" + ((!erfolgreich) ? " nicht" : "") + " erfolgreich. ");
		System.out.printf("Neuer Kontostand = %.2f Euro.", (bs.getKontostand(kontonummer) / 100.0));
	}

	private void kontoauszugDrucken() {
		System.out.print("Bitte die gewünschte Kontonummer für den Auszug eingeben: ");
		int kontonummer = Integer.parseInt(sc.nextLine());

		// in echt auf einem Drucker
		System.out.println("Auszug für Konto " + kontonummer);
		String[] kontobewegungen = bs.erstelleKontoauszug(kontonummer);

		if (kontobewegungen.length > 0)
			for (String kb : kontobewegungen) {
				System.out.println(kb);
			}
		else
			System.out.println("Noch keine Kontobewegungen.");
	}

	private void überweisungBeauftragen() throws Exception {
		System.out.print("Bitte die Kontonummer des Ausgangskontos der Überweisung eingeben: ");
		int startkonto = Integer.parseInt(sc.nextLine());

		System.out.print("Bitte die Kontonummmer für das Zielkonto der Überweisung eingeben: ");
		int zielkonto = Integer.parseInt(sc.nextLine());

		System.out.print("Bitte den gewünschten Überweisungsbetrag eingeben: ");
		double betrag = Double.parseDouble(sc.nextLine());

		System.out.print("Bitte den Verwendungszweck eingeben: ");
		String verwendungszweck = sc.nextLine();

		boolean erfolgreich = bs.überweisungBeauftragen(startkonto, zielkonto, (long) (betrag * 100), verwendungszweck);

		System.out.println("Überweisung" + ((!erfolgreich) ? " nicht" : "") + " erfolgreich ausgeführt.");
	}

	private void saldoAbfragen() {
		System.out.print("Bitte die Kontonummer des gewünschten Kontos eingeben: ");
		int konto = Integer.parseInt(sc.nextLine());

		System.out.print("Bitte die Anzahl der Kontobewegungen für den Saldo eingeben: ");
		int anzahl = Integer.parseInt(sc.nextLine());

		long saldo = bs.saldoBestimmen(konto, anzahl);
		System.out.printf("Der Saldo nach %d Kontobewegungen beträgt %.2f Euro.%n", anzahl, (saldo / 100d));
	}

	private void kundeAnlegen() {
		System.out.println("Vorname");
		String vorname = sc.nextLine();
		System.out.println("Nachname");
		String nachname = sc.nextLine();
		System.out.println("Wohnort");
		String wohnort = sc.nextLine();
		System.out.println("Alter");
		int alter = Integer.parseInt(sc.nextLine());
		int kundennummer = bs.kundeAnlegen(vorname, nachname, wohnort, alter);
		System.out.println("Der Kunde wurde angelegt mit der KN: " + kundennummer);

	}

	private void aktienKaufen() {
		System.out.println("Für welches Depot wollen Sie Aktien kaufen? Bitte Kontonummer angeben: ");
		int kontonummer = Integer.parseInt(sc.nextLine());
		System.out.println(
				"Für welche Firma wollen Sie Aktien ewerben? Zur Auswahl stehen (1) John Deere, (2) Hochschule Mannheim oder (3) Bundeswehr. ");
		int auswahl = Integer.parseInt(sc.nextLine());
		System.out.println("Wie viele Aktien wollen Sie kaufen? ");
		int anzahl = Integer.parseInt(sc.nextLine());

		boolean erfolgreich = bs.aktienKaufen(kontonummer, auswahl, anzahl);
		if (erfolgreich == true)
			System.out.println("Aktienkauf war erfolgreich. ");
		else
			System.out.println("Aktienkauf war nicht erfolgreich. ");

	}

	private void geldEinzahlennDepot() {
		System.out.println("Bitte Kontonummer vom Depot eingeben: ");
		int kontonummer = Integer.parseInt(sc.nextLine());
		System.out.println("Wie viel möchten Sie einzahlen? ");
		long betrag = Long.parseLong(sc.nextLine());

		long neuerStand = bs.geldEinzahlenDepot(kontonummer, betrag);
		System.out.println("Der neue Stand auf dem Depot beträgt: " + neuerStand);

	}

	private void geldAuszahlenDepot() {
		System.out.println("Bitte Kontonummer vom Depot eingeben: ");
		int kontonummer = Integer.parseInt(sc.nextLine());
		System.out.println("Wie viel möchten Sie auszahlen? ");
		long betrag = Long.parseLong(sc.nextLine());

		boolean erfolgreich = bs.geldAuszahlenDepot(kontonummer, betrag);

	}

	private void aktienVerkaufen() {
		System.out.println("Von welchem Depot wollen Sie Aktien verkaufen? Bitte Kontonummer eingeben: ");
		int kontonummer = Integer.parseInt(sc.nextLine());

		System.out.println("Folgende Aktien sind auf dem Konto: ");
		String[] aktien = bs.aktienAnzeigen(kontonummer);

		for (int i = 0; i <= aktien.length - 1; i++) {
			System.out.print(aktien[i] + " ");
		}

		System.out.println(
				"Welche Aktien wollen Sie verkaufen? Zur Auswahl stehen (1) John Deere, (2) Hochschule Mannheim oder (3) Bundeswehr.");
		int auswahl = Integer.parseInt(sc.nextLine());

		System.out.println("Wie viele wollen Sie verkaufen? ");
		int anzahl = Integer.parseInt(sc.nextLine());
		boolean erfolgreich = bs.aktienVerkaufen(kontonummer, auswahl, anzahl);
		if (erfolgreich == true) {
			System.out.println("Der Verkauf war erfolgreich. ");
		} else {
			System.out.println("Der Verkauf war leider nicht erfolgreich. ");
		}

	}

	private void aktienAnzeigen() {
		System.out.println("Von welchem Depot möchten Sie die Aktien sehen? Bitte Kontonummer angeben: ");
		int kontonummer = Integer.parseInt(sc.nextLine());
		String[] aktien = bs.aktienAnzeigen(kontonummer);
		int johnDeereCounter = 0;
		int hsmaCounter = 0;
		int bundeswehrCounter = 0;

		for (int i = 0; i <= aktien.length - 1; i++) {
			if (aktien[i].equals("JohnDeere")) {
				johnDeereCounter++;
			} else if (aktien[i].equals("HSMA")) {
				hsmaCounter++;
			} else if (aktien[i].equals("Bundeswehr")) {
				bundeswehrCounter++;
			}
		}
		System.out.println("JohnDeere = " + johnDeereCounter);
		System.out.println("HSMA = " + hsmaCounter);
		System.out.println("Bundeswehr = " + bundeswehrCounter);

	}

}
